/*******************************************************************************
* FILE NAME: timer_calculator.c
*
* DESCRIPTION:
*  Calculates the various timer settings that will produce a given timer increment
*  This dumps the results to printf
*  User calls timer_calculator(period, allowable_error);
*  where period = Hz and allowable_error = +/- Hz
*  For milliseconds Hz = 1000/#ms
*
*  Desired time = Timer offset / prescale / postscale
*
*  Timer variations:
* 8-bit		255	
* 16-bit		65535
* Timer Ticks		Prescaler				Postscaler
*     0	16-bit 	1:1,2,4,8,16,32,64,128,256		none
*	1	16-bit	1:1,2,4,8						none
*	2	8-bit	1:1,4,16							1:1-16
*	3	16-bit	1:1,2,4,8						none
*	4	8-bit	1:1,4,16							1:1-16
*
*******************************************************************************/

#include <stdio.h>
#include "ifi_default.h"

#define CLOCK 10000000  //10MHz is the PIC clock we use

void timer_0_calculator(void);
void timer_2or4_calculator(void);
void timer_1or3_calculator(void);
void PrintFloat(float);


/*******************************************************************************
* FILE NAME: timer_calculator.c
*
* DESCRIPTION:
*	10MHz / (offset ticks) / (prescale factor) / (postscale factor) = time elapsed 
*	DESIRED_PERIOD	e.g. 2400   			// In Hz
*	DESIRED_PERIOD	e.g. 4ms =(1000/4)	// In ms (1000/n = Hz)
*	WINDOW			e.g.	20	  			// Also in Hz
*******************************************************************************/
 int prescale_0[9]={1,2,4,8,16,32,64,128,256};
 int prescale_1[4]={1,2,4,8};
 int prescale_2[3]={1,4,16};
 int prescale_3[4]={1,2,4,8};
 int prescale_4[3]={1,4,16};
unsigned int desired_period, window;

void timer_calculator(unsigned int period, unsigned int allowable_error)
{
  static char once=0;
  float ms;

  desired_period = period;
  window = allowable_error;

  if (0==once)
  {
	once = 1;
	ms = 1000/period;
	printf("Example of using timers to get %uHz (", period);
	PrintFloat(ms);
	printf("ms) or a one shot at that rate\r\r");
  	timer_2or4_calculator();
	timer_1or3_calculator();
	timer_0_calculator();
	printf("\rCOMPLETED\r");
  }
}


void timer_0_calculator()
{
	long offset;
	long period;
	int pre_i;

	printf("\r----------------------------------------------------------------------\r");
	printf("Timer 0\r");
	printf("----------------------------------------------------------------------\r");
	printf("This  timer can be setup as an 8-bit (offset<256) or 16-bit timer depending\r");
	printf(" on the offset required. Following the less-is-more principle an 8-bit timer is\r");
	printf(" preferred as more efficient, as long as it fits your need.\r");
	printf("CAUTION: Used as an 8-bit timer - if you read the timer value you must use\r");
	printf(" trim off the high-order byte\r");
	printf("e.g.,\r");
	printf("OpenTimer0(TIMER_INT_ON &\r");
	printf("           T0_16BIT &\r");
	printf("           T0_SOURCE_INT &\r");
	printf("           T0_PS_1_2\r");  //Set this to your chosen prescale
	printf("           );\r");
	printf("    WriteTimer0(offset);\r");  // Set this to your chosen offset
	printf("\r");
	printf("DESIRED_PERIOD=%d, WINDOW=%d, Acceptable error=%d percent\r\r", desired_period, window, (int)window*100/desired_period);
	printf("period   offset   prescale   \r");
	printf("-----------------------------\r");

	for (offset=1; offset<65536; offset++) // in units of 10MHz
	{
		if ((CLOCK/offset) < (desired_period-window))
		{
			break; // There will be no more solutions
		}
		if (statusflag.NEW_SPI_DATA)
		{
  	  		Getdata(&rxdata);
			Putdata(&txdata);
		}
		for (pre_i=0; pre_i<9; pre_i++)
		{
			period = (long)CLOCK / offset / prescale_0[pre_i];
			if ((period < (desired_period+window)) && (period > (desired_period-window)))
			{
			   	printf(" %d,      ", (int)period);
				printf("%lu",(65535-offset));
			   	printf(",       %d\r", (int)prescale_0[pre_i]);
			}
		}
	}
}

void timer_1or3_calculator()
{
	long offset;
	long period;
	int pre_i;

	printf("\r----------------------------------------------------------------------\r");
	printf("Timer 1 or Timer 3\r");
	printf("----------------------------------------------------------------------\r");
	printf("These two timers get set to overflow (& wrap around) at the time you want\r");
	printf("WriteTimer1() must be called to reset the value every time in the interrupt\r");
	printf("handler. Some accuracy is lost - the time it takes to reset the timer.\r");
	printf("This  timer can be setup as an 8-bit (offset<256) or 16-bit timer depending\r");
	printf(" on the offset required. Following the less-is-more principle an 8-bit timer is\r");
	printf(" preferred as more efficient, as long as it fits your need.\r");
	printf("CAUTION: Used as an 8-bit timer - if you read the timer value you must use\r");
	printf(" trim off the high-order byte\r");
	printf("e.g.,\r");
	printf("OpenTimer1(TIMER_INT_ON &\r");
	printf("           T1_16BIT_RW &\r");
	printf("           T1_SOURCE_INT &\r");
	printf("           T1_PS_1_2\r");  //Set this to your chosen prescale
	printf("           );\r");
	printf("    WriteTimer1(offset);\r");  // Set this to your chosen offset
	printf("\r");
	printf("DESIRED_PERIOD=%d, WINDOW=%d, Acceptable error=%d percent\r\r", desired_period, window, (int)window*100/desired_period);
	printf("period   offset   prescale   \r");
	printf("-----------------------------\r");

	for (offset=1; offset<65536; offset++) // in units of 10MHz
	{
		if ((CLOCK/offset) < (desired_period-window))
		{
			break; // There will be no more solutions
		}
		if (statusflag.NEW_SPI_DATA)
		{
  	  		Getdata(&rxdata);
			Putdata(&txdata);
		}
		for (pre_i=0; pre_i<4; pre_i++)
		{
			period = (long)CLOCK / offset / prescale_1[pre_i];
			if ((period < (desired_period+window)) && (period > (desired_period-window)))
			{
				printf(" %d,      ", (int)period);
				printf("%lu",(65535-offset));
				printf(",       %d\r", (int)prescale_1[pre_i]);
			}
		}
	}
}


void timer_2or4_calculator()
{
	int offset;
	long period;
	int pre_i, postscale;

	printf("\r----------------------------------------------------------------------\r");
	printf("Timer 2 or Timer 4\r");
	printf("----------------------------------------------------------------------\r");
	printf("These 2 timers have a special Period register PR2 or PR4\r");
	printf("When the timer reaches that value it automatically resets\r");
	printf("This is best when you need an exact, repeatable period\r");
	printf("These don't have to be reset in the interrupt handler\r");
	printf("e.g.,\r");
	printf("  OpenTimer4(TIMER_INT_ON &\r");
	printf("             T4_PS_1_4 &\r");
	printf("             T4_POST_1_7\r");
	printf("             );\r");
	printf("  PR4 = 148;  //2396Hz \r");
	printf("\r");
	printf("DESIRED_PERIOD=%d, WINDOW=%d, Acceptable error=%d percent\r\r", desired_period, window, (int)window*100/desired_period);
	printf("period   offset   prescale   postscale\r");
	printf("--------------------------------------\r");

    for (postscale=1; postscale<16; postscale++)
    {
		if (statusflag.NEW_SPI_DATA)
		{
  	  		Getdata(&rxdata);
			Putdata(&txdata);
		}
		for (offset=1; offset<256; offset++) // in units of 10MHz
		{
			for (pre_i=0; pre_i<3; pre_i++)
			{
				period = (long)CLOCK / offset / prescale_2[pre_i] / postscale;
				if ((period < (desired_period+window)) && (period > (desired_period-window)))
				{
				   	printf(" %d,      %d,       %d,        %d\r", (int)period, (int) (offset-1), (int)prescale_2[pre_i], (int)postscale);
				}
			}
		}
	}
}


/*******************************************************************************
* FUNCTION NAME: PrintFloat
* PURPOSE:       Print floating-point data to the IFI_Loader debug window.
* ARGUMENTS:     
*     Input:odata - char,int, or long to be printed
*******************************************************************************/
#define ACCURACY 100 //How many decimal places are important to you
 
void PrintFloat(float value)
{
	long i, i2; // Don't need to be longs if you only need a few significant digits
 
	/* Separate the whole number from the fraction and print each of them */
 
	i = (long) value;
	i2 = (long) ((value-i)*ACCURACY); // You can get fancy and round by adding .5
 
	printf("%d.%02d", (int)i, (int)i2); //e.g., "234.078"
}
